/****************************************************************************
*
*    Copyright (c) 2005 - 2010 by Vivante Corp.  All rights reserved.
*
*    The material in this file is confidential and contains trade secrets
*    of Vivante Corporation. This is proprietary information owned by
*    Vivante Corporation. No part of this work may be disclosed,
*    reproduced, copied, transmitted, or used in any way for any purpose,
*    without the express written permission of Vivante Corporation.
*
*****************************************************************************
*
*
*****************************************************************************/




#ifndef __gc_egl_h_
#define __gc_egl_h_

#include <EGL/egl.h>
#include <KHR/khrvivante.h>

#include "gc_hal.h"
#include "gc_hal_driver.h"
#include "gc_hal_engine.h"
#include "gc_egl_common.h"

#ifdef __cplusplus
extern "C" {
#endif

#define gcdZONE_EGL_API			(gcvZONE_API_EGL | (1 << 0))
#define	gcdZONE_EGL_SURFACE		(gcvZONE_API_EGL | (1 << 1))
#define	gcdZONE_EGL_CONTEXT		(gcvZONE_API_EGL | (1 << 2))
#define	gcdZONE_EGL_CONFIG		(gcvZONE_API_EGL | (1 << 3))
#define	gcdZONE_EGL_OS			(gcvZONE_API_EGL | (1 << 4))
#define	gcdZONE_EGL_IMAGE		(gcvZONE_API_EGL | (1 << 5))
#define	gcdZONE_EGL_SWAP		(gcvZONE_API_EGL | (1 << 6))
#define	gcdZONE_EGL_INIT		(gcvZONE_API_EGL | (1 << 7))
#define	gcdZONE_EGL_SYNC		(gcvZONE_API_EGL | (1 << 8))

#include "gc_hal_raster.h"

typedef struct eglDisplay		*VEGLDisplay;
typedef struct eglContext		*VEGLContext;
typedef struct eglConfig		*VEGLConfig;
typedef struct eglSurface		*VEGLSurface;
typedef struct eglImage			*VEGLImage;
typedef struct eglSync			*VEGLSync;

#define VEGL_DISPLAY(p)			((VEGLDisplay) (p))
#define VEGL_CONTEXT(p)			((VEGLContext) (p))
#define VEGL_CONFIG(p)			((VEGLConfig)  (p))
#define VEGL_SURFACE(p)			((VEGLSurface) (p))
#define VEGL_IMAGE(p)			((VEGLImage)   (p))
#define VEGL_SYNC(p)			((VEGLSync)    (p))

#define EGL_DISPLAY_SIGNATURE	gcmCC('E','G','L','D')
#define EGL_SURFACE_SIGNATURE	gcmCC('E','G','L','S')
#define EGL_CONTEXT_SIGNATURE	gcmCC('E','G','L','C')
#define EGL_IMAGE_SIGNATURE		gcmCC('E','G','L','I')
#define EGL_SYNC_SIGNATURE		gcmCC('E','G','L','Y')

#define veglUSE_HAL_DUMP		0
#define veglRENDER_LIST_LENGTH	5

struct eglRenderList
{
	gctSIGNAL					signal;
	gcoSURF						surface;
	gctPOINTER					bits;

	struct eglRenderList *		prev;
	struct eglRenderList *		next;
};

struct eglWorkerInfo
{
	gctPOINTER					thread;

	gctSIGNAL					signal;
	gctSIGNAL					targetSignal;

	VEGLSurface					draw;
#ifdef EGL_API_ANDROID
	gctPOINTER					logical;
#endif
	void*						bits;

	gctINT						left;
	gctINT						top;
	gctINT						width;
	gctINT						height;
	gctUINT32					offset;

#ifdef ANDROID_VERSION_ECLAIR	
 /* gctPOINTER					nativeBuffer; */
    gctBOOL						skipBlt;
#endif	
    struct eglWorkerInfo *		next;
};

#define _EGL_NUM_BUFFERS_       2

#ifdef EGL_API_ANDROID
#define EGL_WORKER_COUNT		_EGL_NUM_BUFFERS_
#else
#define EGL_WORKER_COUNT		4
#endif

#define _EGL_INIT_CONTEXT_LIST_SIZE         256
#define _EGL_INIT_SURFACE_LIST_SIZE         256


typedef struct _EGL_THREAD_DATA
{
	/* gcoOS object. */
	gcoOS						os;

	/* gcoHAL object. */
	gcoHAL						hal;

	/* gcoDUMP object. */
	gcoDUMP						dump;

	/* Internal memory pool. */
	gctSIZE_T					internalSize;
	gctPHYS_ADDR				internalPhysical;
	gctPOINTER					internalLogical;

	/* External memory pool. */
	gctSIZE_T					externalSize;
	gctPHYS_ADDR				externalPhysical;
	gctPOINTER					externalLogical;

	/* Contiguous memory pool. */
	gctSIZE_T					contiguousSize;
	gctPHYS_ADDR				contiguousPhysical;
	gctPOINTER					contiguousLogical;

	/* Last known error code. */
	EGLenum						error;

	/* Current API. */
	EGLenum						api;
	EGLBoolean					es20;

	/* Hardware capabilities. */
	gceCHIPMODEL				chipModel;
	EGLint						maxWidth;
	EGLint						maxHeight;
	EGLint						maxSamples;
	gctBOOL						openVGpipe;
	gctBOOL						vaa;

	/* Current context. */
	VEGLContext					context;
	EGLint						lastClient;

	/* Allocated resources. */
	VEGLContext					contextStack;
	EGLBoolean					destroying;

	struct eglWorkerInfo *		worker;
	struct eglWorkerInfo		workers[EGL_WORKER_COUNT];

    /* Game Patch  */
    EGLBoolean                  bForce8888;
}
* VEGLThreadData;

/*!
*********************************************************************
*   \struct recursiveMutex
*   \brief
*       Data structure for recursive lock.
*********************************************************************
*/
struct recursiveMutex
{
	/* Thread lock the mutex. */
    VEGLThreadData				pThread;
	/* Lock times. */
    EGLint                      nReference;
	/* Access mutex. */
	gctPOINTER					accMutex;
	/* Underly mutex. */
	gctPOINTER					undMutex;
};



typedef struct eglImageRef *	VEGLImageRef;
struct eglImageRef
{
	VEGLImageRef				next;
	EGLNativePixmapType			pixmap;
	gcoSURF						surface;
};

struct eglDisplay
{
	/* Signature. */
	gctUINT						signature;

	/* Next EGLDisplay. */
	VEGLDisplay					next;

	/* Handle to device context. */
	NativeDisplayType			hdc;
	gctBOOL						releaseDpy;

	/* Process handle. */
	gctHANDLE					process;

	/* Number of configurations. */
	EGLint						configCount;

	/* Pointer to configurations. */
	VEGLConfig					config;

	/* Allocated resources. */
	VEGLSurface	*				surfaceList;
	EGLint						nSurfaceCount;
	EGLint						nSurfaceListSize;
	VEGLContext	*				contextList;
	EGLint						nContextCount;
	EGLint						nContextListSize;
	VEGLImage					imageStack;
	VEGLImageRef				imageRefStack;
	/* Reference counter. */
	gcsATOM_PTR					reference;

	/* Worker thread for copying data. */
	gctHANDLE					workerThread;
	gctSIGNAL					startSignal;
	gctSIGNAL					stopSignal;
	gctPOINTER					suspendMutex;
	gcoOS						os;

	/* Locking mutex. */
	struct recursiveMutex		*recMutex;

	struct eglWorkerInfo *		worker;

	gctCHAR						exeName[64];
    
};

struct eglConfig
{
	EGLint						bufferSize;
	EGLint						alphaSize;
	EGLint						blueSize;
	EGLint						greenSize;
	EGLint						redSize;
	EGLint						depthSize;
	EGLint						stencilSize;
	EGLenum						configCaveat;
	EGLint						configId;
	EGLBoolean					defaultConfig;
	EGLBoolean					nativeRenderable;
	EGLint						nativeVisualType;
	EGLint						samples;
	EGLint						sampleBuffers;
	EGLenum						surfaceType;
	EGLBoolean					bindToTetxureRGB;
	EGLBoolean					bindToTetxureRGBA;
	EGLint						luminanceSize;
	EGLint						alphaMaskSize;
	EGLenum						colorBufferType;
	EGLenum						renderableType;
	EGLenum						conformant;
	EGLenum						matchFormat;
	EGLint                      matchNativePixmap;
	EGLint						width;
	EGLint						height;
};

struct eglSurface
{
	/* Signature. */
	gctUINT						signature;

	/* Next EGLSurface. */
	VEGLSurface					next;

	/* Set if VG pipe is present and this surface was created when the current
	   API was set to OpenVG. */
	gctBOOL						openVG;

	/* Swap surface. */
	gcoSURF						swapSurface;
	gctUINT						swapBitsPerPixel;

	/* Direct frame buffer access. */
	gcoSURF						fbWrapper;
	gctBOOL						fbDirect;
	gctPOINTER					fbLogical;
	gctUINT32					fbPhysical;
	gctINT32					fbStride;
	gctINT32					fbWidth;
	gctINT32					fbHeight;

	/* Render target array. */
	gctBOOL						renderListEnable;
	struct eglRenderList *		renderListHead;
	struct eglRenderList *		renderListCurr;
	gctUINT						renderListCount;

	/* Render target. */
	gcoSURF						renderTarget;
	void *						renderTargetBits;
	gceSURF_FORMAT				renderTargetFormat;
	gcsSURF_FORMAT_INFO_PTR		renderTargetInfo[2];

	/* Depth buffer. */
	gcoSURF						depthBuffer;
	gceSURF_FORMAT				depthFormat;

	/* Resolve surface. */
	gcoSURF						resolve;
	void *						resolveBits;
	gctUINT						resolveBitsPerPixel;

	/* Bitmap data. */
	gctINT						bitsPerPixel;
	gctINT						bitsX;
	gctINT						bitsY;
	gctINT						bitsWidth;
	gctINT						bitsHeight;
	gctUINT						bitsAlignedWidth;
	gctUINT						bitsAlignedHeight;
	gctINT						bitsStride;

	/* Surface Config Data. */
	struct eglConfig			config;

	/* Reference counter. */
	gctBOOL						created;
	gcsATOM_PTR					reference;

	/* Surface type. */
	EGLint						type;
	EGLenum						buffer;
	gceSURF_COLOR_TYPE			colorType;

	/* Window attributes. */
	NativeWindowType			hwnd;

	/* PBuffer attributes. */
	EGLBoolean					largestPBuffer;
	EGLBoolean					mipmapTexture;
	EGLint						mipmapLevel;
	EGLenum						textureFormat;
	EGLenum						textureTarget;

	/* Pixmap attributes. */
	NativePixmapType			pixmap;

	gctINT						pixmapStride;
	gctPOINTER					pixmapBits;

	/* Lock surface attributes. */
	EGLBoolean					locked;
	gcoSURF						lockBuffer;
	void *						lockBits;
	gceSURF_FORMAT				lockBufferFormat;
	gctINT						lockBufferStride;
	EGLBoolean					lockPreserve;

    /* Android specific members */
	gctPOINTER					module;

	/* gctPOINTER					currentNative; */
	gctBOOL						currentSkipBlt;

	EGLBoolean					skipBlt;
	struct
	{
		EGLBoolean              mapped;
		gctPOINTER              mappingInfo;
		gctUINT32               physical;
		gctPOINTER              memory;        
		gctUINT32               size;
        gctUINT32               width;
        gctUINT32               height;
        gctUINT32               stride;
        gceSURF_FORMAT          format;
    	gctPOINTER				nativeBuffer;
        gctBOOL                 inUse;

	}  virtualMap[_EGL_NUM_BUFFERS_];
    
    EGLint                      prevMapIndex;    
    EGLint                      curMapIndex;
    EGLint                      nextMapIndex;

    gctBOOL						mapInvalid;
    
    EGLBoolean                  bFullMapped;

    gctUINT                     swapCount;
    gctUINT                     lastSwapTime;

    EGLBoolean                  bCompositor;
    EGLBoolean                  b2DPath;
    EGLBoolean                  b2DPathDither;    
    EGLBoolean                  bSingleThread;
    
    EGLBoolean                  b2DPathSmart;
    EGLBoolean                  bNeedDither; 
    EGLBoolean                  bDitherSwitch;
    gctUINT                     noDitherFrames;
    
    gcsRECT                     dirtyRect;
    
    gcsRECT                     dirtyRegion;
    gcsRECT                     oldDirtyRegion;

	/* Pointer to current context. */
	VEGLContext					pCurrentContext;

};

struct eglImage
{
	/* Internal image struct pointer. */
	khrEGL_IMAGE				image;

	/* Signature. */
	gctUINT						signature;

	/* Next eglImage. */
	VEGLImage					next;
};

struct eglContext
{
	/* Signature. */
	gctUINT						signature;

	/* Next EGLContext. */
	VEGLContext					next;

	/* Bounded thread. */
	VEGLThreadData				thread;

	/* Bounded API. */
	EGLenum						api;
	EGLint						client;

	/* Context dispatch table for API. */
	veglDISPATCH *				dispatch;

	/* Attached display. */
	VEGLDisplay					display;

	/* Shared configuration. */
	VEGLContext					sharedContext;

	/* Current read/draw surface */
	VEGLSurface					read;
	VEGLSurface					draw;

	/* Context for API. */
	void *						context;

	/* Mark for destroy */
	EGLBoolean					bDestroyed; 					    
};

struct eglSync
{
	/* Signature. */
	gctUINT						signature;

	/* Type */
	EGLenum						type;

	/* Signal */
	gctSIGNAL					signal;
};

/*!
*********************************************************************
*   \struct  _EGL
*   \brief
*       Data structure for _EGL
*********************************************************************
*/
typedef struct _EGL
{
	/* Display stack. */
	VEGLDisplay					displayStack;
	/* Recursive mutex. */
	struct recursiveMutex		*recMutex;
}_EGL_T;

/* Global resources. */
extern gcoOS g_os;
extern _EGL_T* g_pEGL;

typedef enum _VEGL_COLOR_FORMAT
{
	VEGL_DEFAULT	= (1 << 0),
	VEGL_ALPHA		= (1 << 1),
	VEGL_444		= (1 << 2),
	VEGL_555		= (1 << 3),
	VEGL_565		= (1 << 4),
	VEGL_888		= (1 << 5),
	VEGL_YUV		= (1 << 6),

	VEGL_4444		= VEGL_ALPHA | VEGL_444,
	VEGL_5551		= VEGL_ALPHA | VEGL_555,
	VEGL_8888		= VEGL_ALPHA | VEGL_888
}
VEGL_COLOR_FORMAT;

typedef struct EGL_CONFIG_COLOR
{
	EGLint						bufferSize;

	EGLint						redSize;
	EGLint						greenSize;
	EGLint						blueSize;
	EGLint						alphaSize;

	VEGL_COLOR_FORMAT			formatFlags;
}
* VEGLConfigColor;

typedef struct EGL_CONFIG_DEPTH
{
	int depthSize;
	int stencilSize;
}
* VEGLConfigDepth;


/*******************************************************************************
** Thread API functions.
*/

VEGLThreadData
veglGetThreadData(
	void
	);

void
veglDestroyThreadData(
	VEGLThreadData Thread
	);

#ifdef EGL_API_ANDROID
void veglClearThreadData(
        VEGLThreadData Thread
        );
#endif

/*******************************************************************************
** Display API functions.
*/

EGLBoolean
veglReferenceDisplay(
	IN VEGLThreadData Thread,
	IN VEGLDisplay Display
	);

void
veglDereferenceDisplay(
	IN VEGLThreadData Thread,
	IN VEGLDisplay Display,
	EGLBoolean Always
	);


/*******************************************************************************
** Surface API functions.
*/

EGLBoolean
veglReferenceSurface(
	IN VEGLThreadData Thread,
	IN VEGLSurface Surface
	);

EGLBoolean
veglDereferenceSurface(
	IN VEGLThreadData Thread,
	IN VEGLSurface Surface,
	IN EGLBoolean Always
	);

gceSTATUS
veglAddRenderListSurface(
	IN VEGLThreadData Thread,
	IN VEGLSurface Surface
	);

gceSTATUS
veglFreeRenderList(
	IN VEGLThreadData Thread,
	IN VEGLSurface Surface
	);

gceSTATUS
veglSetDriverTarget(
	IN VEGLThreadData Thread,
	IN VEGLSurface Surface
	);

gceSTATUS
veglDriverSurfaceSwap(
	IN VEGLThreadData Thread,
	IN VEGLSurface Surface
	);

EGLint
veglResizeSurface(
	IN VEGLSurface Surface,
	IN gctUINT Width,
    IN gctUINT Height,
	gceSURF_FORMAT ResolveFormat,
	gctUINT BitsPerPixel
	);

/*******************************************************************************
** eglImage API functions.
*/
void
veglDestroyImage(
	IN VEGLThreadData Thread,
	IN VEGLDisplay Display,	IN VEGLImage Image
	);


/*******************************************************************************
** Swap worker API functions.
*/

void
veglSuspendSwapWorker(
	VEGLDisplay Display
	);

void
veglResumeSwapWorker(
	VEGLDisplay Display
	);

#if defined(_WIN32)
#	define veglTHREAD_RETURN DWORD
#else
#	define veglTHREAD_RETURN void *
#endif

#ifndef WINAPI
#   define WINAPI
#endif

veglTHREAD_RETURN WINAPI
veglSwapWorker(
	void* Argument
	);

gctHANDLE
veglGetModule(
	gcoOS Os,
	EGLBoolean Egl,
	VEGLContext Context
	);


/*******************************************************************************
** Bridge functions.
*/

veglDISPATCH *
_GetDispatch(
    VEGLThreadData Thread,
	VEGLContext Context
    );

void *
_CreateApiContext(
    VEGLThreadData Thread,
	VEGLContext Context,
    gcoOS Os,
    gcoHAL Hal,
	void * SharedContext
    );

EGLBoolean
_DestroyApiContext(
    VEGLThreadData Thread,
	VEGLContext Context,
    void * ApiContext
    );

EGLBoolean
_FlushApiContext(
    VEGLThreadData Thread,
	VEGLContext Context,
    void * ApiContext
    );

EGLBoolean
_SetApiContext(
    VEGLThreadData Thread,
	VEGLContext Context,
    void * ApiContext,
    gcoSURF Draw,
    gcoSURF Read,
    gcoSURF Depth,
    void * Priv 
    );

gceSTATUS
_SetBuffer(
	VEGLThreadData Thread,
	gcoSURF Draw
	);

EGLBoolean
_Flush(
	VEGLThreadData Thread
	);

EGLBoolean
_Finish(
    VEGLThreadData Thread
    );

EGLBoolean
_eglProfileCallback(
    VEGLThreadData Thread,
    IN gctUINT32 Enum,
    IN gctUINT32 Value
    );

gcoSURF
_GetClientBuffer(
    VEGLThreadData Thread,
    void * Context,
	EGLClientBuffer buffer
    );

EGLenum
_BindTexImage(
	VEGLThreadData Thread,
	gcoSURF Surface,
	EGLenum Format,
	EGLenum Target,
	EGLBoolean Mipmap,
	EGLint Level
	);

EGLenum
_CreateImageTexture(
	VEGLThreadData Thread,
	EGLenum Target,
	gctINT Texture,
	gctINT Level,
	gctINT Depth,
	gctPOINTER Image
	);

EGLenum
_CreateImageFromRenderBuffer(
	VEGLThreadData Thread,
	gctUINT Framebuffer,
	gctPOINTER Image
	);

EGLenum
_CreateImageFromVGParentImage(
	VEGLThreadData	Thread,
	unsigned int	vgimage_obj,
	VEGLImage		eglimage
	);


EGLBoolean
_eglAddDisplay (
	VEGLDisplay dpy
	);

VEGLDisplay
_eglGetDisplayStack ();


EGLBoolean
_eglRemoveContextFromDisplay (
	VEGLDisplay dpy,
	VEGLContext ctx
	);

EGLBoolean
_eglRemoveSurface(
	VEGLSurface sur
	);

EGLBoolean
_eglAddContextToDisplay (
	VEGLDisplay dpy,
	VEGLContext ctx
	);

EGLBoolean
_eglAddSurfaceToDisplay (
	VEGLDisplay dpy,
	VEGLSurface sur
	);

EGLBoolean
_eglIsValidDisplay (
	VEGLDisplay dpy
	);

EGLBoolean
_eglIsValidContext(
	VEGLDisplay dpy,
	VEGLContext ctx
	);

EGLBoolean
_eglIsValidSurface (
	VEGLDisplay dpy,
	VEGLSurface sur
    );

EGLBoolean
_initEGL(void);

void
_TerminateDisplayStack(void);

void
_releaseEGL(void);

EGLBoolean
_eglRecursiveLock(
	struct recursiveMutex *pRecMutex
    );

EGLBoolean
_eglRecursiveUnlock(
	struct recursiveMutex *pRecMutex
    );

void
_eglThreadLock(void);

void
_eglThreadUnlock(void);


void
_eglDisplayLock(
	VEGLDisplay pDisplay
    );

void
_eglDisplayUnlock(
	VEGLDisplay pDisplay
    );

typedef enum _NativePixelFormat
{
    NATIVE_PIXFMT_ARGB8888  = 0,
    NATIVE_PIXFMT_RGBA8888  = 1,
    NATIVE_PIXFMT_ARGB1555  = 2,
    NATIVE_PIXFMT_RGBA5551  = 3,
    NATIVE_PIXFMT_RGB565    = 4,

    NATIVE_PIXFMT_ABGR8888  = 5,

    NATIVE_PIXFMT_RGB888    = 6,
}NativePixelFormat;


typedef enum _NATIVE_POWER_EVENT{
    NATIVE_POWER_STANDBY,
    NATIVE_POWER_HIBERNATE,
    NATIVE_POWER_RESUME,
}_NATIVE_POWER_EVENT_T;

typedef enum _PIXEL_FORMAT
{
    /* 32 bits */
    _PIXEL_FORMAT_lARGB8888 = 0,
    _PIXEL_FORMAT_lARGB8888_PRE,

    _PIXEL_FORMAT_sARGB8888,
    _PIXEL_FORMAT_sARGB8888_PRE,

    _PIXEL_FORMAT_lRGBA8888,
    _PIXEL_FORMAT_lRGBA8888_PRE,

    _PIXEL_FORMAT_sRGBA8888,
    _PIXEL_FORMAT_sRGBA8888_PRE,


    _PIXEL_FORMAT_sABGR8888,
    _PIXEL_FORMAT_sABGR8888_PRE,


    _PIXEL_FORMAT_D32,
    _PIXEL_FORMAT_D24S8,

    /* 24 bits */
    _PIXEL_FORMAT_D24,
    _PIXEL_FORMAT_sRGB888,

    /* 16 bits */
    _PIXEL_FORMAT_sARGB1555,    
    _PIXEL_FORMAT_sRGBA5551,  

    _PIXEL_FORMAT_sRGB565,

    _PIXEL_FORMAT_lLA88,
    _PIXEL_FORMAT_lLA88_PRE,

    _PIXEL_FORMAT_sLA88,    
    _PIXEL_FORMAT_sLA88_PRE,    

    _PIXEL_FORMAT_D16,

    /* 8 bits */
    _PIXEL_FORMAT_lL8,
    _PIXEL_FORMAT_sL8,

    _PIXEL_FORMAT_A8,
    
    _PIXEL_FORMAT_S8,
    
    _PIXEL_FORMAT_BW8,  /* Though BW1 originally is 1bit, we internally store it as 8bit  */

    _PIXEL_FORMAT_LAST,
}_PIXEL_FORMAT_T;

typedef enum _EGL_SYNC_TYPE
{
    _EGL_SYNC_FROM_TARGET,
    _EGL_SYNC_TO_TARGET,
}_EGL_SYNC_TYPE_T;

const char* _eglBoolName(EGLBoolean value);

const char* _eglSyncName(_EGL_SYNC_TYPE_T type);

const char* _eglEnumName(EGLenum value);

const char* _nativeFmtName(EGLenum value);

const char* _nativeEventName(_NATIVE_POWER_EVENT_T value);

const char* _esmFmtName(EGLenum value);


#ifdef __cplusplus
}
#endif

#endif /* __gc_egl_h_ */

